@@ -44,9 +44,9 @@ class AegisHandler(tornado.web.RequestHandler):
         valid_subdomains = aegis.config.get('valid_subdomains')
         if len(host_split) > 2 and valid_subdomains and host_split[0] not in valid_subdomains:
             self.tmpl['host'] = '.'.join(host_split[1:])
-        # Ignore crazy hostnames. Only use the ones we have specified.
+        # Ignore hostnames not in config.py. Only use the ones we have specified.
         if self.tmpl['host'] not in config.hostnames.keys():
-            logging.warning("Ignore crazy hostname: %s", self.tmpl['host'])
+            logging.warning("Ignore hostname not specified in config.py: %s", self.tmpl['host'])
             raise tornado.web.HTTPError(404)
         config.apply_hostname(self.tmpl['host'])
         self.tmpl['options'] = options
@@ -132,6 +132,9 @@ class AegisHandler(tornado.web.RequestHandler):
             self.tmpl['user']['user_id'] = user['user_id']
 
     def user_is_robot(self):
+        # If this fails so early that setup_user() doesn't run, just parse the user_agent here
+        if not self.tmpl.get('user_agent_obj'):
+            self.tmpl['user_agent_obj'] = user_agents.parse(self.tmpl['user_agent'])
         return bool(self.tmpl['user_agent_obj'].is_bot or aegis.stdlib.is_robot(self.tmpl['user_agent']))
 
     def render(self, template_name, **kwargs):
@@ -268,8 +271,9 @@ class AegisHandler(tornado.web.RequestHandler):
             return None
         if 'member' not in self.tmpl:
             self.get_current_user()
-        if self.tmpl['member']:
+        if self.tmpl['member'] and self.tmpl['member'].get('email'):
             return self.tmpl['member']['email']['email']
+        logging.error("No email for this member. Does get_member_email() need to be overridden in a subclass? Is self.get_current_user() overridden in a subclass?")
 
     def del_current_user(self):
         self.cookie_clear('auth')
@@ -592,6 +596,14 @@ class AegisHydra(AegisWeb):
         return self.redirect(self.request.uri)
 
 
+class AegisHydraQueue(AegisWeb):
+    @tornado.web.authenticated
+    def get(self, *args):
+        self.enforce_admin()
+        self.tmpl['hydra_queues'] = aegis.model.HydraQueue.scan()
+        return self.render_path("hydra_queue.html", **self.tmpl)
+
+
 class AegisReportForm(AegisWeb):
 
     def validate_report_type(self, report_type_id):
